#ifndef MC2HEP_H
#define MC2HEP_H

#include <HepMC/GenEvent.h>
#include <HepMC/HeavyIon.h>
#include <HepMC/HEPEVT_Wrapper.h>
#include <HepMC/IO_GenEvent.h>
#include <HepMC/IO_HEPEVT.h>
#include <HepMC/PdfInfo.h>

#ifdef HEPMC_HAS_CROSS_SECTION
#include <HepMC/GenCrossSection.h>
#endif

#ifdef HEPMC_HAS_UNITS
#include <HepMC/Units.h>
#endif

#include <iomanip>
#include <iostream>
#include <sstream>
#include <stdexcept>
#include <stdlib.h>
#include <string>

#ifndef __WITHOUTBOOST__
#include <boost/iostreams/device/file_descriptor.hpp>
#include <boost/iostreams/filtering_stream.hpp>
#include <boost/iostreams/filter/gzip.hpp>
#include <boost/iostreams/filter/zlib.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/path.hpp>
#else
#include <fstream>
#endif

extern "C"
{
  void crmc_f_( int&, double&, int&, double&, double&, double&,
                double&, double&);
};

extern "C"
{
  void crmc_init_f_( int&, double&, double&, int&, int&, int&, const char*);
};

extern "C"
{
  void crmc_xsection_f_(double&, double&, double&, double&, double&, double&, double&, double&, double&);
};

extern "C"
{
  extern struct
  {
    float sigtot;
    float sigcut;
    float sigela;
    float sloela;
    float sigsd;
    float sigine;
    float sigdif;
    float sigineaa;
    float sigtotaa;
    float sigelaaa;
    float sigcutaa;
  } hadr5_; //crmc-aaa.f
}

extern "C"
{
  extern struct
  {
    // nevt .......... error code. 1=valid event, 0=invalid event
    // bimevt ........ absolute value of impact parameter
    // phievt ........ angle of impact parameter
    // kolevt ........ number of collisions
    // koievt ........ number of inelastic collisions
    // pmxevt ........ reference momentum
    // egyevt ........ pp cm energy (hadron) or string energy (lepton)
    // npjevt ........ number of primary projectile participants
    // ntgevt ........ number of primary target participants
    // npnevt ........ number of primary projectile neutron spectators
    // nppevt ........ number of primary projectile proton spectators
    // ntnevt ........ number of primary target neutron spectators
    // ntpevt ........ number of primary target proton spectators
    // jpnevt ........ number of absolute projectile neutron spectators
    // jppevt ........ number of absolute projectile proton spectators
    // jtnevt ........ number of absolute target neutron spectators
    // jtpevt ........ number of absolute target proton spectators
    // xbjevt ........ bjorken x for dis
    // qsqevt ........ q**2 for dis
    // sigtot ........ total cross section
    // nglevt ........ number of collisions acc to  Glauber
    // zppevt ........ average Z-parton-proj
    // zptevt ........ average Z-parton-targ
    // ng1evt ........ number of Glauber participants with at least one IAs
    // ng2evt ........ number of Glauber participants with at least two IAs
    // ikoevt ........ number of elementary parton-parton scatterings
    // typevt ........ type of event (1=Non Diff, 2=Double Diff, 3=Single Diff
    float phievt;
    int nevt;
    float bimevt;
    int kolevt;
    int koievt;
    float pmxevt;
    float egyevt;
    int npjevt;
    int ntgevt;
    int npnevt;
    int nppevt;
    int ntnevt;
    int ntpevt;
    int jpnevt;
    int jppevt;
    int jtnevt;
    int jtpevt;
    float xbjevt;
    float qsqevt;
    int nglevt;
    float zppevt;
    float zptevt;
    int minfra;
    int maxfra;
    int kohevt;
  } cevt_; //epos.inc
}

extern "C"
{
  extern struct
  {
    int ng1evt;
    int ng2evt;
    float rglevt;
    float sglevt;
    float eglevt;
    float fglevt;
    int ikoevt;
    float typevt;
  } c2evt_; //epos.inc
}

bool   GetOptions(int, char**);
void   cleanVector();
void   PrintTestEvent();

// --------------  default options
int    gNCollision=2;
int    gFirstEventNumber=0;
int    gSeed=123;
int    gBeamID=1;
int    gTargetID=1;
int    gHEModel=0;
double gBeamMomentum=3500;
double gTargetMomentum=-3500;  //-gBeamMomentum*82./208.; //LHC magnet design for Pb
std::string gParamFileName="crmc.param";

std::string gOutfileName="example_hep.hepmc";
bool   gTest=false;
bool   gCompression=false;

// --------------  Tree data
const unsigned int gMaxParticles = 9990; // HEP max number of particles MUST BE EQUAL TO THE NUMBER SET IN crmc-aaa.f

int          gNParticles = 0;
double  gImpactParameter = -1.;
static int   gPartID[gMaxParticles];
static double gPartPx[gMaxParticles];
static double gPartPy[gMaxParticles];
static double gPartPz[gMaxParticles];
static double gPartEnergy[gMaxParticles];
static double gPartMass[gMaxParticles];

// --------------  test observables
double TotalEnergy=0. ;
double Multiplicity=0. ;
double MeanPseudorapidity=0. ;
double PlateauHeight=0. ;
double MeanPt=0. ;
double ErrorTotalEnergy=0. ;
double ErrorMultiplicity=0. ;
double ErrorMeanPseudorapidity=0. ;
double ErrorPlateauHeight=0. ;
double ErrorMeanPt=0. ;
double xsigtot   ;
double xsigine   ; 
double xsigela   ; 
double xsigdd    ;
double xsigsd    ;
double xsloela   ;
double xsigtotaa ;
double xsigineaa ;
double xsigelaaa ;


#endif //ifdef MC2HEP_H
